import TypeUtils from "./utils/type"

import ActionsFormsSubmitV1 from "./actions/forms/submit"
import ActionsFormsEncodeV1 from "./actions/forms/encode"

import ActionsHttpPostV1 from "./actions/http/post"
import ActionsHttpPatchV1 from "./actions/http/patch"
import ActionsHttpDeleteV1 from "./actions/http/delete"

import ActionsDialogsAlertV1 from "./actions/dialogs/alert"
import ActionsDialogsOpenV1 from "./actions/dialogs/open"
import ActionsDialogsOptionsV1 from "./actions/dialogs/options"
import ActionsDialogsSnackbarV1 from "./actions/dialogs/snackbar"

import ActionsWindowsCloseAllV1 from "./actions/windows/closeAll"
import ActionsWindowsOpenV1 from "./actions/windows/open"
import ActionsWindowsOpenWebV1 from "./actions/windows/openWeb"

import ActionsAuthSaveCsrfTokenV1 from "./actions/auth/saveCsrfToken"
import ActionsAuthRestartV1 from "./actions/auth/restart"

const actions = {
  "forms/submit-v1": ActionsFormsSubmitV1,
  "forms/encode-v1": ActionsFormsEncodeV1,

  "http/post-v1": ActionsHttpPostV1,
  "http/patch-v1": ActionsHttpPatchV1,
  "http/delete-v1": ActionsHttpDeleteV1,

  "dialogs/alert-v1": ActionsDialogsAlertV1,
  "dialogs/open-v1": ActionsDialogsOpenV1,
  "dialogs/options-v1": ActionsDialogsOptionsV1,
  "dialogs/snackbar-v1": ActionsDialogsSnackbarV1,

  "windows/closeAll-v1": ActionsWindowsCloseAllV1,
  "windows/open-v1": ActionsWindowsOpenV1,
  "windows/openWeb-v1": ActionsWindowsOpenWebV1,

  "auth/saveCsrfToken-v1": ActionsAuthSaveCsrfTokenV1,
  "auth/restart-v1": ActionsAuthRestartV1,
};

export default class Action {
  static execute(spec, target, controller, params = {}) {
    if (!TypeUtils.isObject(spec)) {
      return
    }

    const name = spec["action"]
    if (TypeUtils.isString(name)) {
      console.log(`Executing "${name}"`)

      try {
        const action = new actions[name]()
        action.execute(Object.assign(spec, params), target, controller)
      }
      catch (e) {
        console.log(`Failed executing command: "${e.message}"`)
      }
    }
    else {
      console.log(`Invalid action: "${name}"`)
    }
  }
}

window.Action = Action
