import { DirectUpload } from "@rails/activestorage"

export default class Uploader {
  constructor(input, file, url) {
    this.input = input
    this.file = file
    this.url = url
    this.upload = new DirectUpload(this.file, this.url, this)
  }

  start(callback) {
    this.dispatch("start")
    this.upload.create((error, blob) => callback(error, blob))
  }

  dispatchEvent(element, type, eventInit = {}) {
    const { disabled } = element
    const { bubbles, cancelable, detail } = eventInit
    const event = document.createEvent("Event")

    event.initEvent(type, bubbles || true, cancelable || true)
    event.detail = detail || {}

    try {
      element.disabled = false
      element.dispatchEvent(event)
    } finally {
      element.disabled = disabled
    }

    return event
  }

  dispatch(name, detail = {}) {
    detail.file = this.file
    return this.dispatchEvent(this.input, `direct-upload:${name}`, { detail })
  }

  directUploadWillStoreFileWithXHR(request) {
    request.upload.addEventListener("progress", event => this.directUploadDidProgress(event))
  }

  directUploadDidProgress(event) {
    const progress = event.loaded / event.total * 100
    if (progress)
      this.dispatch("progress", { progress })
  }
}
